#include "mainwindow.h"
#include "ui_mainwindow.h"
#include "myglframe.h"
#include "myXmlStreamReader.h"
#include <iostream>
#include <QtDebug>
#include <QtGui>
#include <QString>
//#include <QString>

MainWindow::Vessel MainWindow::v;
QList<MainWindow::Vessel> MainWindow::vList;
int MainWindow::maxVeinWidth;

MainWindow::MainWindow(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::MainWindow)
{

    ui->setupUi(this);
    GLFrame = new MyGLFrame();      // create our subclassed GLWidget
    setCentralWidget(GLFrame);      // assign it to the central widget of the window
    GLFrame->initRenderThread();    // start rendering
    init();
}

MainWindow::~MainWindow()
{
    delete GLFrame;
    delete ui;
}

void MainWindow::init()
{
    createActions();
    createMenus();
    setWindowIcon(QIcon(":/images/cg_footer_klecks.gif"));

//    MyXmlStreamReader MyReader;
//    QString fileName = "c:/sample.xml";
//    MyReader.readFile(fileName);
//    createVesselTreeStructure(MyReader);

}

void MainWindow::createMenus()
{
    fileMenu = menuBar()->addMenu(tr("&File"));
    fileMenu->addAction(openAction);
    fileMenu->addSeparator();
    fileMenu->addAction(exitAction);

    helpMenu = menuBar()->addMenu(tr("&Help"));
    helpMenu->addAction(helpAction);
    helpMenu->addAction(aboutQtAction);
}

void MainWindow::createActions()
{
    openAction = new QAction(tr("&Open File"),this);
    openAction->setIcon(QIcon(":/images/film-go.png"));
    openAction->setShortcut(tr("Ctrl+O"));
    openAction->setStatusTip(tr("Open XML File with Vessels data"));
    connect(openAction, SIGNAL(triggered()),this,SLOT(parseXML()));

    exitAction = new QAction(tr("&Quit"),this);
    exitAction->setIcon(QIcon(":/images/film-go.png"));
    exitAction->setShortcut(tr("Ctrl+Q"));
    exitAction->setStatusTip(tr("Close Application"));
    connect(exitAction,SIGNAL(triggered()),this,SLOT(close()));

    helpAction = new QAction(tr("Technical Support"),this);
    helpAction->setIcon(QIcon(":/images/film-go.png"));
    helpAction->setStatusTip(tr("Informations library's"));
    //connect(helpAction,SIGNAL(triggered()),this,SLOT(createVesselTreeStructure())); // now just for debuging

    aboutQtAction = new QAction(tr("About &Qt"),this);
    aboutQtAction->setIcon(QIcon(":/images/film-go.png"));
    aboutQtAction->setStatusTip(tr("Show the Qt library's About box"));
    connect(aboutQtAction,SIGNAL(triggered()),qApp,SLOT(aboutQt()));

}


bool MainWindow::okToContinue()
{
    if (isWindowModified()) {
        int r = QMessageBox::information(this, tr("Vis2Vene"),
        tr("Do you want to open new XML data file?"),
        QMessageBox::Yes | QMessageBox::No
        | QMessageBox::Cancel);
        if (r == QMessageBox::Yes) {
            return true;
        } else if (r == QMessageBox::Cancel) {
            return false;
        }
    }
    return true;
}

void MainWindow::parseXML()
{
    QString fileName = QFileDialog::getOpenFileName(
                             this,
                             tr("Open File"), ".",
                             tr("Vessel-Tree files (*.xml)")
                             );
    if (!fileName.isEmpty()) {
        MyXmlStreamReader MyReader;
        //MyReader = new MyXmlStreamReader();
        MyReader.readFile(fileName);
        createVesselTreeStructure(MyReader);
    }

    return;
}

void MainWindow::createVesselTreeStructure (MyXmlStreamReader &MyReader){

    QTextStream cout(stdout);

//    if (MyReader.cpMap->size()!= 0)
//    cout<<"Number of control points is " << MyReader.cpMap->size() <<endl;
//    if (MyReader.segmentList->size() !=0)
//    cout<<"Number of segments is " << MyReader.segmentList->size() <<endl;

    // print connection segment - control points
    QList<Segment>::const_iterator itCP;
    QList<Segment> *list = MyReader.segmentList;
//    for(itCP = list->begin(); itCP!=list->end(); ++itCP){
//        cout << itCP->BeginCPID << ":" << itCP->EndCPID << " ";
//     }

    // odredjivanje pocetnih segmenata
    QList<Segment> sviPocetniSegmenti;
    Segment tmpS;
    Segment tmpE;
    int countVein = 0;
    for (int k = 0 ; k < list->size(); k++) {
        //cout << endl << "Vein "<< k+1 << ": " << tmpS.BeginCPID << ":" << tmpS.EndCPID << " \nSve:";
        bool isNew = true;
        tmpS = list->at(k);
        for ( int i = 0; i < list->size() ; i++) {
            tmpE = list->at(i);
            //cout << " " << tmpS.BeginCPID << ":" << tmpS.EndCPID ;
            if (tmpS.BeginCPID == tmpE.EndCPID || (tmpS.BeginCPID == tmpE.BeginCPID && i != k)) {
                  isNew = false;
            }
        }
        if (isNew) {
            //cout << "Start vene: " << tmpS.BeginCPID << ":" << tmpS.EndCPID << endl;
            countVein++;
            sviPocetniSegmenti.push_back(tmpS);

        }
    }
   // cout << "Ukupno vena: " << countVein << "   U listi: " << sviPocetniSegmenti.size() << endl;




     //read all control points and find edge_degree
     itCP = MyReader.segmentList->begin();
     Segment seg = *itCP;


     ID = 0; // brench ID - for one vein "die Ader"
     int startPoint = 1; //  the vein start; each brench gets its own new startPoint,
                         //  depending on the number of points previously in vein
     Circumference = 0;
     testSegmentCount = 0;
     vList.clear();
     //maxVeinWidth_cur = 0;
     for (int i = 0; i < sviPocetniSegmenti.size(); i++)  {
     v = createVessel(sviPocetniSegmenti.at(i), MyReader, startPoint, Circumference);
     ID = 0;
     Circumference = 0;
     startPoint = 1;
     //maxVeinWidth_cur = 0;
     vList.push_back(v);
     }
     //cout << "Ukupno segemenata iz rekurzije: " <<      testSegmentCount  << endl;

 }//createVesselTreeStructure


MainWindow::Vessel MainWindow::createVessel(Segment start, MyXmlStreamReader &MyReader, int startPoint, float Circumference){
    QTextStream cout(stdout);
//    cout << "calling create vessel" << endl;
    Vessel *v = new Vessel();
    ID++;
    v->id=ID;


    v->startPoint = startPoint; // start point for this branche
    v->segments= new QList<Segment>();
    v->branches = new QList<Vessel>();
    int startAt=0;
    Segment s =start;
    float maxCircumference = Circumference;
    v->maxCircumference=maxCircumference;

    while (true) {
        testSegmentCount++;
        v->segments->push_back(s);
        // count number of points in all segments before branch occur
        // this number is input paremeter for new branche
        // increment by the number of points in current segment
        startAt += s.pList->size();
        // check maximum circumference of all points in current segment
        float newMax = getCircumferenceFromSegment(s);
        if( newMax > maxCircumference) {
            maxCircumference = newMax;
            v->maxCircumference = newMax;
        }

   //     std::cout << " v->segments->push_back(s) " << s.BeginCPID << ":" << s.EndCPID << std::endl;
        QList<Segment> list = getSegmentsBegin(s.EndCPID, MyReader);
        if (list.size()==2) {
       //  cout << " list.size()==2 ";
            if (list[0].EndCPID == s.EndCPID+1) {
                Vessel v1 = createVessel(list[1], MyReader, startAt, Circumference);
                s=list[0];
                //cout << " v->branches->push_back(v1) = end+1 " << v1.id << endl ;
                v->branches->push_back(v1);

            } else {
                Vessel v1 = createVessel(list[0], MyReader, startAt, Circumference);
                s=list[1];
                //cout << " v->branches->push_back(v1) " << v1.id << endl;
                v->branches->push_back(v1);

            }
        } else if (list.size()==1) {
            //cout << " list.size()==1 " << list[0].BeginCPID << ":" << list[0].EndCPID << endl;
            s = list[0];
        } else {
            break;
        }
    }
    return *v;
}


QList<MainWindow::Segment>MainWindow::getSegmentsBegin(int beginID, MyXmlStreamReader &MyReader) {

    QList<Segment> *l = new QList<Segment>();
    QList<Segment>::iterator it;
    for(it = MyReader.segmentList->begin(); it!=MyReader.segmentList->end(); ++it){
        if (it->BeginCPID == beginID) {
            l->push_back(*it);
        }
    }
    return *l;
}

float MainWindow::getCircumferenceFromSegment(Segment s){
    QList<DataStructure::Point> *pL = s.pList;
    QList<DataStructure::Point>::Iterator it;

    float max = 0.0f;
    for(it = pL->begin(); it!= pL->end(); ++it ){
        DataStructure::Point p = *it;
        float newMax = p.r1;
        if (newMax > max)
            max = newMax;
    }
    return max;
}


void MainWindow::printVessel(Vessel v,QTextStream &cout, QString s) {
//    cout << "\n";
//    cout << s << "ID=" << v.id << " Segmentcount: " << v.segments->size() << "\n";
    QList<Vessel>::const_iterator itCP;
    QList<Vessel> list = *v.branches;
//    cout << s<< "Branching Vessels: \n";
    for(itCP = list.begin(); itCP!=list.end(); ++itCP){

        //create some memory to read fields into
        printVessel(*itCP, cout, s+"--");
    // cout <<":"<< *itCP->EndCPID << endl; // MyReader.controlPoint *cpo = new MyReader.controlPoint;
    }
 //   cout << s<< "End branching vessels \n";

}
