#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include <QMainWindow>
#include "myXmlStreamReader.h"

class QAction;
class FindDialog;
class MyGLFrame;

namespace Ui {
    class MainWindow;
}

// We define the class MainWindow as a subclass of QMainWindow. It contains the Q_OBJECT
// macro since it provides its own signals and slots.
class MainWindow : public QMainWindow , public DataStructure
{
    Q_OBJECT // The Q_OBJECT macro at the beginning of the class definition is necessary
             // for all classes that define signals or slots.

public:
    explicit MainWindow(QWidget *parent = 0);
        // The MainWindow constructor is typical of Qt widget classes. The parent parameter
        // specifies the parent widget. The default is a null pointer, meaning that the
        // dialog has no parent.

    ~MainWindow();

    /** Vessel is a struct for the main vein and the branching veins
     * @brief The Vessel struct
     */
    struct Vessel {
        int id;
        QList<Segment> *segments;
        QList<Vessel> *branches;
        int startPoint;
        float maxCircumference;
    };

    static Vessel v;
    /** list of all veins
     * @brief vList
     */
    static QList<Vessel> vList;
    static int maxVeinWidth;
    int maxVeinWidth_cur;
    int ID;
    float Circumference;
    int testSegmentCount;

    /** from segments and control point it creates on or more veins
     * @brief createVesselTreeStructure
     * @param MyReader
     */
    void createVesselTreeStructure(MyXmlStreamReader &MyReader);

    /**
     * @brief createVessel
     * @param start
     * @param MyReader
     * @param startAt branching points
     * @param Circumference for each vein
     * @return
     */
    Vessel createVessel(Segment start, MyXmlStreamReader &MyReader, int startAt, float Circumference);

    /**
     * @brief getSegmentsBegin
     * @param beginID
     * @param MyReader
     * @return
     */
    QList<Segment> getSegmentsBegin(int beginID, MyXmlStreamReader &MyReader);

    /**
     * @brief printVessel
     * @param v
     * @param cout
     * @param s
     */
    void printVessel(Vessel v, QTextStream &cout, QString s);

    /**
     * @brief getCircumferenceFromSegment
     * @param s
     * @return
     */
    float getCircumferenceFromSegment(Segment s);


//protected:
    /*The closeEvent() function is a virtual function in QWidget that is automatically called
     *when the user closes the window. It is reimplemented in MainWindow so that we can ask
     *the user the standard question "Do you want to save your changes?" and to save user
     *preferences to disk.
     */
 //   void closeEvent(QCloseEvent *event);

private:

    Ui::MainWindow *ui;
    MyGLFrame *GLFrame;

    void init();
    void createMenus();
    void createActions();
    bool okToContinue();
    bool loadFile(const QString &fileName);

    QAction *openAction;
    QAction *exitAction;
    QAction *helpAction;
    QAction *aboutQtAction;

    QMenu *fileMenu;
    QMenu *helpMenu;

    // The slots/signals keyword is actually a macro. The C++ preprocessor converts
    // it into standard C++ before the compiler sees it.

private slots:
    /** parse xml file
     * @brief parseXML
     */
    void parseXML();
    //void about();
};

#endif // MAINWINDOW_H
