#include "myXmlStreamReader.h"
#include <QtGui>
#include <iostream>
#include <QtDebug>
#include <QString>

bool MyXmlStreamReader::readFile(QString &fileName)
{
    //.
    QFile file(fileName);

  //  QTextStream cout(stdout); //just for debuging

    //test to see if the file is readable and that it contains text.
    bool open = file.open(QIODevice::ReadOnly | QIODevice::Text);
    if (!open)
    {
        //        QMessageBox::critical(this,
        //                                     "MyXmlStreamReader::readFile",
        //                                     "Couldn't open example.xml",
        //                                     QMessageBox::Ok);

        qDebug() << "Couldn't open file";
        return false;
    }
    else
    {
        qDebug() << "File opened OK " << file.fileName();
    }

    //set device
    xml.setDevice(&file);

    cpMap = new std::map<int, controlPoint>();
    segmentList = new QList<Segment>();

    while (!xml.atEnd() ) {
        QXmlStreamReader::TokenType token = xml.readNext();
        if(token == QXmlStreamReader::Invalid) {
            std::cerr << "invalid token";
        }
        /* Test to see if the token indicates that we’re at the beginning
            of the document. Any calls to the reader’s current data will
            give you the xml version number information. */
        if(token == QXmlStreamReader::StartDocument) {
            //qDebug() <<"LineNumber: " << xml.lineNumber();
            //cout << " StartDocument token "  << endl;
            continue;
        }

        /* If token is StartElement, we'll see if we can read it.*/
        if(token == QXmlStreamReader::StartElement) {
            //If it's named vesseltree, we'll go to the next.
            if(xml.name() == "vesseltree") {
                continue;
            }
            //Header element. We need its children
            if(xml.name() == "header") {
                this->readHeaderElement(xml);
            }
            //Cps itself doesn’t do anything for us. We need its
            //children, so just skip this element*/
            if(xml.name() == "cps") {
                continue;
            }
            /*cp element. We need its children*/
            if(xml.name() == "cp") {
                this->readCpElement(xml);
            }
            //segments element itself doesn’t do anything for us. We need its
            //children, so just skip this element*/
            if(xml.name() == "segments") {
                continue;
            }
            /*segment element. We need this element and its children*/
            if(xml.name() == "segment") {
                this->readSegmentElement(xml);
            }
        }//startElement
    }//while

    // Error handling
    if(xml.hasError()) {
 //       std::cerr << "Error: Failed to parse file "
   //               << qPrintable(fileName) << ": "
     //             << qPrintable(xml.errorString()) << std::endl;
        return false;
    }
 //   std::cout << " DONE with parsing XML file... "  << std::endl;
    xml.clear();
   // cout<<"Number of control points is: "<<cpMap->size()<<endl;
   // cout<<"Number of segments is: "<<segmentList->size()<<endl;
    return true;
}//readFile


void MyXmlStreamReader::readHeaderElement(QXmlStreamReader& xml)
{
  //  QTextStream cout(stdout);
    //check to ensure that we were called in the appropriate spot!
    if(xml.tokenType() != QXmlStreamReader::StartElement
            && xml.name() != "header"){
        qDebug() << "Called XML readHeaderElement "
                 << "without a header element in XML stream!";
                    return;
    }

    // We need to get data from headers's children...
    xml.readNext();

    /*And now the code looks very similar to that of the caller!
      the way to read the while condition is as follows:
      while it isn’t the case that we're at the end of an element and
      (to protect against nested questions) we're not looking at a
      new header element*/
    while(!(xml.tokenType() == QXmlStreamReader::EndElement
          && xml.name() == "header")){

        //at the start of an element, otherwise ignore and
        //keep reading.
        if(xml.tokenType() == QXmlStreamReader::StartElement){

            //If the element is a nx element, save it
            if(xml.name() == "nx"){
            xml.readNext();
            header.nx = xml.text().toString().toInt();
            }
            //If the element is a ny element, save it
            if(xml.name() == "ny"){
            xml.readNext();
            header.ny = xml.text().toString().toInt();
            }
            //If the element is a nz element, save it
            if(xml.name() == "nz"){
            xml.readNext();
            header.nz = xml.text().toString().toInt();
            }
            //If the element is a px element, save it
            if(xml.name() == "px"){
            xml.readNext();
            header.px = xml.text().toString().toFloat();
            }
            //If the element is a py element, save it
            if(xml.name() == "py"){
            xml.readNext();
            header.py = xml.text().toString().toFloat();
            }
            //If the element is a py element, save it
            if(xml.name() == "pz"){
            xml.readNext();
            header.pz = xml.text().toString().toFloat();
            }
            //otherwise we don’t know what it is, so do nothing,
            //and read the next thing!
        }
        //we’re currently looking at some element feld,
        //so read the next element header!
        xml.readNext();
        //cout<<xml.name().toString()<< endl;
     }//while
    return;
}//readHeaderElement


void MyXmlStreamReader::readCpElement(QXmlStreamReader& xml)
{
    //check to ensure that we were called in the appropriate spot!
    if(xml.tokenType() != QXmlStreamReader::StartElement
            && xml.name() != "cp"){
        qDebug() << "Called XML readHeaderElement "
                 << "without a cp element in XML stream!";
                    return;
    }

    // We need to get data from cp's children...
    xml.readNext();

    //create some memory to read fields into
    controlPoint *cp = new controlPoint;

    while(!(xml.tokenType() == QXmlStreamReader::EndElement
          && xml.name() == "cp")){

        if(xml.tokenType() == QXmlStreamReader::StartElement){

            if(xml.name() == "id"){
            xml.readNext();
            cp->ID = xml.text().toString().toFloat();
            }

            if(xml.name() == "x"){
            xml.readNext();
            cp->x = xml.text().toString().toFloat();
            }

            if(xml.name() == "y"){
            xml.readNext();
            cp->y = xml.text().toString().toFloat();
            }

            if(xml.name() == "z"){
            xml.readNext();
            cp->z = xml.text().toString().toFloat();
            }

            if(xml.name() == "r1"){
            xml.readNext();
            cp->r1 = xml.text().toString().toFloat();
            }

            if(xml.name() == "r2"){
            xml.readNext();
            cp->r2 = xml.text().toString().toFloat();
            }

            if(xml.name() == "offset"){
            xml.readNext();
            cp->offset = xml.text().toString().toFloat();
            }

            //otherwise we don’t know what it is, so do nothing,
            //and read the next thing!
        }
        //we’re currently looking at some element feld,
        //so read the next element header!
        xml.readNext();        
     }//while  
    cpMap->insert(std::pair<int, controlPoint>(cp->ID, *cp));
    return;
}//readCpElement


void MyXmlStreamReader::readSegmentElement(QXmlStreamReader& xml)
{
    //check to ensure that we were called in the appropriate spot!
    if(xml.tokenType() != QXmlStreamReader::StartElement
            && xml.name() != "segment"){
        qDebug() << "Called XML readSegmentElement "
                 << "without a segment element in XML stream!";
                    return;
    }

    // We need to get data from segment's children...
    xml.readNext();

    //create some memory to read fields into
    Segment *seg = new Segment;
    QList<Point> *list= new QList<Point>();
    seg->pList=list;

    while(!(xml.tokenType() == QXmlStreamReader::EndElement
          && xml.name() == "segment")){

        //at the start of an element, otherwise ignore and
        //keep reading.
        if(xml.tokenType() == QXmlStreamReader::StartElement){

            if(xml.name() == "BeginCPID"){
            xml.readNext();
            seg->BeginCPID= xml.text().toString().toInt();
            }

            if(xml.name() == "EndCPID"){
            xml.readNext();
            seg->EndCPID = xml.text().toString().toInt();
            }

            if(xml.name() == "VesselType"){
            xml.readNext();
            seg->VesselType = xml.text().toString().toInt();
            }

            if(xml.name() == "VesselState"){
            xml.readNext();
            seg->VesselState = xml.text().toString().toInt();
            }

            if(xml.name() == "Description"){
            xml.readNext();
            seg->Description = xml.text().toString();
            }

            if(xml.name() == "points"){
            this->readPointsElement(xml,*seg);
            }

            //otherwise we don’t know what it is, so do nothing,
            //and read the next thing!
        }
        //we’re currently looking at some element feld,
        //so read the next element header!
        xml.readNext();       
     }//while    
    segmentList->push_back(*seg);
    return;
}//readSegmentElement

void MyXmlStreamReader::readPointsElement(QXmlStreamReader &xml, Segment &seg) {
    if(xml.tokenType() != QXmlStreamReader::StartElement
       && xml.name() != "points"){
            qDebug() << "Called XML readPointsElement "
                     << "without a points element in XML stream!";
            return;
    }

    // We need to get data from points children...
    xml.readNext();

    while(!(xml.tokenType() == QXmlStreamReader::EndElement
          && xml.name() == "points")){

        //at the start of an element, otherwise ignore and
        //keep reading.
        if(xml.tokenType() == QXmlStreamReader::StartElement){

            //If the element is a point element, we need its children
            if(xml.name() == "p"){
            this->readPointElement(xml, seg);
            }

            //otherwise we don’t know what it is, so do nothing,
            //and read the next thing!
        }       
        //we’re currently looking at some element feld,
        //so read the next element header!
        xml.readNext();
     }//while
}//readPointsElement


void MyXmlStreamReader::readPointElement(QXmlStreamReader &xml, Segment &seg) {
    if(xml.tokenType() != QXmlStreamReader::StartElement
       && xml.name() != "p"){
            qDebug() << "Called XML readPointElement "
                     << "without a point element in XML stream!";
            return;
    }
    // We need to get data from segment's children...
    xml.readNext();

    //create some memory to read fields into
    Point *point = new Point;

    while(!(xml.tokenType() == QXmlStreamReader::EndElement
          && xml.name() == "p")){

        //at the start of an element, otherwise ignore and
        //keep reading.
        if(xml.tokenType() == QXmlStreamReader::StartElement){

            if(xml.name() == "x"){
                xml.readNext();
                point->x = xml.text().toString().toFloat();
            }

            if(xml.name() == "y"){
                xml.readNext();
                point->y = xml.text().toString().toFloat();
            }

            if(xml.name() == "z"){
                xml.readNext();
                point->z = xml.text().toString().toFloat();
            }

            if(xml.name() == "r1"){
                xml.readNext();
                point->r1 = xml.text().toString().toFloat();
            }

            if(xml.name() == "r2"){
                xml.readNext();
                point->r2 = xml.text().toString().toFloat();
            }

            if(xml.name() == "offset"){
                xml.readNext();
                point->offset = xml.text().toString().toFloat();
            }

            //otherwise we don’t know what it is, so do nothing,
            //and read the next thing!
        }       
        //we’re currently looking at some element feld,
        //so read the next element header!
        xml.readNext();
    }//while
    seg.pList->push_back(*point);
    return;
}//readPointElement

/*
void MyXmlStreamReader::skipUnknownElement()
{

    xml.readNext();
    while (!xml.atEnd()) {
        if (xml.isEndElement()) {
            xml.readNext();
            break;
        }

        if (xml.isStartElement()) {
            skipUnknownElement();
        } else {
            xml.readNext();
        }
    }
}
*/
