#ifndef MYXMLSTREAMREADER_H
#define MYXMLSTREAMREADER_H

#include <QTreeView>
#include <QTreeWidget>
#include <map>
#include <QtXml/QXmlStreamReader>

/**
 * @brief The DataStructure class
 */
class DataStructure{

    public:
    // protected:

        struct Header{
            int nx; int ny; int nz;
            float px; float py; float pz;
            } header;

        struct Point{
            float x; float y; float z;
            float r1; float r2; float offset;
        };

        struct Segment{
            QString type;
            int BeginCPID;
            int EndCPID;
            int VesselType;
            int VesselState;
            QString Description;
            int pointsCount;
            QList <Point> *pList;
        };

        struct controlPoint{
            int ID;
            float x; float y; float z;
            float r1; float r2; float offset;

        };

        /** contains all segments from xml file
         * @brief segmentList
         */
        QList<Segment> *segmentList;

        /** contains all control points from xml file
         * @brief cpMap
         */
        std::map<int, controlPoint> *cpMap;
};


class MyXmlStreamReader : public DataStructure{

    public:
        //MyXmlStreamReader();

         /** create a new file object with the XML file path, set device and call xml stream reader to read file
         * @brief readFile
         * @param fileName
         * @return
         */
        bool readFile(QString &fileName);

        /** parses an <header>...</header> element
         * @brief readHeaderElement
         * @param xml
         */
        void readHeaderElement(QXmlStreamReader& xml);

        /** parses an <cp>...</cp> element
         * @brief readCpElement
         * @param xml
         */
        void readCpElement(QXmlStreamReader& xml); //

        /** parses an <segment>...</segment> element
         * @brief readSegmentElement
         * @param xml
         */
        void readSegmentElement(QXmlStreamReader& xml); //

        /** parses an <points>...</points> element
         * @brief readPointsElement
         * @param xml
         * @param seg
         */
        void readPointsElement(QXmlStreamReader& xml, Segment &seg); //

        /** parses an <point>...</point> element
         * @brief readPointElement
         * @param xml
         * @param seg
         */
        void readPointElement(QXmlStreamReader& xml, Segment &seg); //

        QXmlStreamReader xml;

};

#endif // MYXMLSTREAMREADER_H
