#include "myrenderthread.h"
#include "myglframe.h"
#include "mainwindow.h"
#include <iostream>
#include <QLabel>

MyRenderThread::MyRenderThread(MyGLFrame *_GLFrame) :
    QThread(),
    GLFrame(_GLFrame)
{
    doRendering = true;
    doResize = false;
    FrameCounter=0;

}

void MyRenderThread::resizeViewport(const QSize &size)
{
    w = size.width();

    h = size.height();
    doResize = true;

}

void MyRenderThread::stop()
{
    doRendering = false;
}


void MyRenderThread::run()
{
    GLFrame->makeCurrent();
    GLInit();

    while (doRendering)
    {
        if (doResize)
        {
            GLResize(w, h, 1000);
            doResize = false;
        }
        glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
        glLoadIdentity();


        //sizeWidth=paintGL(); // render actual frame

        paintGL(); // render actual frame


        FrameCounter++;
        GLFrame->swapBuffers();

        msleep(16); // wait 16ms => about 60 FPS
    }
}


void MyRenderThread::GLInit(void)
{
    glClearColor(1.0f, 1.0f, 1.0f, 0.0f);     // Background => dark blue

    Color c1;
    c1.R = 239; c1.G = 138; c1.B = 98;
    colorList.push_back(c1);

    Color c2;
    c2.R = 153; c2.G = 153; c2.B = 153;
    colorList.push_back(c2);

    Color c3;
    c3.R = 216; c3.G = 179; c3.B = 101;
    colorList.push_back(c3);

    Color c4;
    c4.R = 255; c4.G = 165; c4.B = 0;
    colorList.push_back(c4);

    Color c5;
    c5.R = 219; c5.G = 112; c5.B = 147;
    colorList.push_back(c5);

    Color c6;
    c6.R = 119; c6.G = 136; c6.B = 153;
    colorList.push_back(c6);

    Color c7;
    c7.R = 050; c7.G = 205; c7.B = 050;
    colorList.push_back(c7);

    Color c8;
    c8.R = 188; c8.G = 143; c8.B = 143;
    colorList.push_back(c8);


    Color c9;
    c9.R = 252; c9.G = 141; c9.B = 89;
    colorList.push_back(c9);

    Color c10;
    c10.R = 147; c10.G = 112; c10.B = 219;
    colorList.push_back(c10);

}


void MyRenderThread::GLResize(int width, int height, int sizeX)
{
//    std::cout<<"w"<<width<<std::endl;
//    std::cout<<"h"<<height<<std::endl;
    glViewport(0, 0, width, height);

    glMatrixMode(GL_PROJECTION);

    glLoadIdentity();
    //gluPerspective(45.,((GLfloat)width)/((GLfloat)height),0.1f,1000.0f);

    glOrtho(0, 1.0, 1.0, 0, -1.0, 1.0);

    // ovako da sadrzaj se ne povecava sa velicinom prozora, tj. da bude pixel na monitoru jednak pixelu iscrtanom( odprilike laicki :-) )
    //glScalef((float)1, (float)1, 0.0);
    // povecava se sadrzaj prozora
    //float scaleX = (width / sizeX);
    glScalef((float)1/2500, (float)1/500, 0.0);
    //glScalef((float)1/500, (float)1/4000, 0.0);
    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();


}


void MyRenderThread::paintGL(void){

    glLoadIdentity();

    // TODO error handling if the vein list is empty
    if (MainWindow::vList.size() < 1 )
        return;

    int lastyPosY = 0;

    for( int i = 0 ; i < MainWindow::vList.size(); i++) {
        int offsetStart = 1;
        for (int j =0; j<15; j++){
            leveloffsets[j] = offsetStart;
            offsetStart += 50;
        }

        MainWindow::Vessel tempVessel = MainWindow::vList.at(i);

        float maxCircumference = getMaximumCircumference(tempVessel);

        int startingLevel = 0;
        int connectiontoFather = 0; //remember where is my start x point

        // last position y is used for global tracking, and is used as start y pos when drawing next vein
        lastyPosY = paintVessels(tempVessel,maxCircumference,startingLevel, lastyPosY, connectiontoFather);

//        int tmpY = lastyPosY+50;

//        // now draw line to separate new vesseltree
//        glLineWidth(4.0f);
//        glBegin(GL_LINES);
//        glColor3ub( 80, 80, 80 );
//        glVertex3f(0,tmpY,0);
//        glVertex3f(500,tmpY,0);
//        glEnd();

        lastyPosY+=50;

    }//for each vein

}//paintGL


int MyRenderThread::paintVessels(MainWindow::Vessel v, float  maxc, int level, int startingY, int connectionToFather){

    int offsetToMaxc = 70;
    int y = (qRound((maxc+offsetToMaxc))*(level+1))+startingY; //starting y. (level+1

    int x = leveloffsets[level]; //starting x position for the current level of the tree;
    int midBetweenTwoLevels = midBetweenTwoLevels = qRound( (y-((maxc+offsetToMaxc)*(level-1)+startingY)) /4); // (levelCurrent).y - (levelCurrent-1).y / 4;

    //Draw connecting line from the vein to the firs level branche
    if (leveloffsets[level] > 1){
       // vertical line upwards from the middle of the vein to the middle between two veins
        glLineWidth(2.0f);
        glBegin(GL_LINES);
        Color color = colorList[v.id];
        glColor3ub( color.R, color.G, color.B );
        glVertex3f(x,(y-midBetweenTwoLevels),0);
        glVertex3f(x,y,0);
        glEnd();
        // connecting it to the branch position
        glLineWidth(2.0f);
        glBegin(GL_LINES);
        glColor3ub( color.R, color.G, color.B );
        //glColor3ub( 0, 0, 0 );
        glVertex3f(x,y-midBetweenTwoLevels,0); // start of the branch
        if(connectionToFather<x){
            glVertex3f(connectionToFather-2,y-midBetweenTwoLevels,0); // go to the branching point
        }
        else{
            glVertex3f(connectionToFather+2,y-midBetweenTwoLevels,0); // go to the branching point
        }
        glEnd();
    }


    QList<DataStructure::Segment>::const_iterator itS;
    QList<DataStructure::Segment> *sL = v.segments;
    for(itS = sL->begin(); itS != sL->end(); ++itS){

        QList<DataStructure::Point>::const_iterator itP;
        QList<DataStructure::Point> *pL = itS->pList;
        for (itP = pL->begin(); itP != pL->end(); ++itP) {

            DataStructure::Point p = *itP;
            float radius  = p.r1;

            glLineWidth(2.0f);
            glBegin(GL_LINES);
            glColor3ub( 250, 128, 114 );
            glColor3ub( 100, 100, 100 );
            glVertex3f(x,y,0);
            glVertex3f(x,y+abs(radius),0);
            glEnd();
            // mirroring
            glBegin(GL_LINES);
            glColor3ub( 100, 100, 100 );
            glVertex3f(x,y,0);
            glVertex3f(x,y-abs(radius),0);
            glEnd();

            x+= 1;
        }//for pont list

    }//for segment list

    // x offset for painting current branch. First branch in the level starts always at pos x=1
    int currentOffsetX = leveloffsets[level];

    // save offset for next branch in the same level. x is the last x position
    // separate branches of the same level by move next branch some n pixels on the right
    leveloffsets[level] = x+50;

    // for tracking gloabal y position
    int lastyPos = y;

    QList<MainWindow::Vessel>::const_iterator itV;
    QList<MainWindow::Vessel> listV = *v.branches;
    for(itV = listV.begin(); itV != listV.end(); ++itV){

        // start of subranch
        int xPosL = itV->startPoint; // this is start point of the current branch within the father vein.
                                     // this position is saved during reading of the xml file.
                                     // we need now to add offset to xPosL to move it to the right
                                     // to ensure that xPosL is drawn correctly in the father vein
        int xTranslated = xPosL + currentOffsetX;

        // visualise point in the father vein where the branch is connected to the vein
        glLineWidth(2.0f);
        glBegin(GL_LINES);
        Color c = colorList[(itV->id)];
        glColor3ub( c.R, c.G, c.B );
        glVertex3f( xTranslated, y,              0);
        glVertex3f( xTranslated, y+midBetweenTwoLevels ,0);
        glEnd();

        int connectionToBrach = xTranslated;
        int branchyPos = paintVessels(*itV, maxc, level+1,startingY, connectionToBrach);
        // tracking of the y coordinate to move it down
        if (branchyPos>lastyPos)
            lastyPos = branchyPos;
    }

    // return last y position so that we know, if there are other veins, y position to draw new vein
    return lastyPos;
}


float MyRenderThread::getMaximumCircumference(MainWindow::Vessel v) {
    float max = v.maxCircumference;
    for( int i = 0 ; i < v.branches->size(); i++) {
        MainWindow::Vessel temp = v.branches->at(i);
        //Recursive call
        float tempmax=getMaximumCircumference(temp);

        if (tempmax>max)
            max=tempmax;
    }
    return max;
}
