#ifndef MYRENDERTHREAD_H
#define MYRENDERTHREAD_H

#include <QThread>
#include "mainwindow.h"

class MyGLFrame;
class QSize;

class MyRenderThread : public QThread
{
    Q_OBJECT
public:

    /**
     * @brief MyRenderThread
     * @param parent
     */
    explicit MyRenderThread(MyGLFrame *parent = 0);

    /**
     * @brief resizeViewport
     * @param size
     */
    void resizeViewport(const QSize &size);

    /**
     * @brief run
     */
    void run(void);

    /**
     * @brief stop
     */
    void stop(void);    

    /** Maximum circumference of the current vein is used to avoid
     * overlapping of branches when drawing on the screen
     * @brief getMaximumCircumference
     * @param v - current vein
     * @return maximum Circumference for current vein
     */
    float getMaximumCircumference(MainWindow::Vessel v);

    /**
     * @brief paintVessels
     * @param v
     * @param maxCircumference
     * @param startingLevel
     * @param startingY
     * @return last position y, so that we know, if there are other veins, where to draw new vein
     */
    int paintVessels(MainWindow::Vessel v, float maxCircumference, int startingLevel, int startingY, int connectiontoFather);

protected:
    /**
     * @brief GLInit
     */
    void GLInit(void);

    /**
     * @brief GLResize
     * @param width
     * @param height
     * @param sizeX
     */
    void GLResize(int width, int height, int sizeX);

    /** all painting is done here
     * @brief paintGL
     */
    void paintGL(void);

private:
    bool doRendering, doResize;
    int w, h, FrameCounter;
    MyGLFrame *GLFrame;
    int scale_w;
    int scale_h;
    // leveloffsets[i] holds the current level number. Deepness is set to 15.
    int leveloffsets[15];

    struct Color{
       int R;
       int G;
       int B;
    };

    /** colors used to visualise connections between branches within one vein
     * @brief colorList
     */
    QList<Color> colorList;

    //create a QPainter and pass a pointer to the device.
    //A paint device can be a QWidget, a QPixmap or a QImage
    //QPainter painter(this);
};

#endif // MYRENDERTHREAD_H
